# Generated by Django 3.2.7 on 2023-06-20 14:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('advanced_report_builder', '0010_auto_20230428_2033'),
    ]

    operations = [
        migrations.AddField(
            model_name='barchartreport',
            name='date_field_type',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Single'), (2, 'Range')], default=1),
        ),
        migrations.AddField(
            model_name='barchartreport',
            name='end_date_field',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
    ]
